@@ -98,7 +98,7 @@ static PyObject *py_cleanup(PyObject *self, PyObject *args, PyObject *kwargs)
          // set everything back to input
          if (gpio_direction[gpio] != -1) {
             setup_gpio(gpio, INPUT, PUD_OFF);
-            gpio_direction[i] = -1;
+            gpio_direction[gpio] = -1;
             found = 1;
          }
       }
@@ -363,12 +363,12 @@ static PyObject *py_add_event_callback(PyObject *self, PyObject *args, PyObject
    Py_RETURN_NONE;
 }
 
-// python function add_event_detect(gpio, edge, callback=None, bouncetime=0
+// python function add_event_detect(gpio, edge, callback=None, bouncetime=None)
 static PyObject *py_add_event_detect(PyObject *self, PyObject *args, PyObject *kwargs)
 {
    unsigned int gpio;
    int channel, edge, result;
-   unsigned int bouncetime = 0;
+   int bouncetime = -666;
    PyObject *cb_func = NULL;
    char *kwlist[] = {"gpio", "edge", "callback", "bouncetime", NULL};
 
@@ -399,6 +399,12 @@ static PyObject *py_add_event_detect(PyObject *self, PyObject *args, PyObject *k
       return NULL;
    }
 
+   if (bouncetime <= 0 && bouncetime != -666)
+   {
+      PyErr_SetString(PyExc_ValueError, "Bouncetime must be greater than 0");
+      return NULL;
+   }
+
    if (check_gpio_priv())
       return NULL;
 
@@ -406,7 +412,7 @@ static PyObject *py_add_event_detect(PyObject *self, PyObject *args, PyObject *k
    {
       if (result == 1)
       {
-         PyErr_SetString(PyExc_RuntimeError, "Edge detection already enabled for this GPIO channel");
+         PyErr_SetString(PyExc_RuntimeError, "Conflicting edge detection already enabled for this GPIO channel");
          return NULL;
       } else {
          PyErr_SetString(PyExc_RuntimeError, "Failed to add edge detection");
@@ -481,14 +487,16 @@ static PyObject *py_event_detected(PyObject *self, PyObject *args)
       Py_RETURN_FALSE;
 }
 
-// python function py_wait_for_edge(gpio, edge)
-static PyObject *py_wait_for_edge(PyObject *self, PyObject *args)
+// python function wait_for_edge(channel, edge, bouncetime=None)
+static PyObject *py_wait_for_edge(PyObject *self, PyObject *args, PyObject *kwargs)
 {
    unsigned int gpio;
    int channel, edge, result;
-   char error[30];
+   int bouncetime = -666; // None
 
-   if (!PyArg_ParseTuple(args, "ii", &channel, &edge))
+   static char *kwlist[] = {"channel", "edge", "bouncetime", NULL};
+
+   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "ii|i", kwlist, &channel, &edge, &bouncetime))
       return NULL;
 
    if (get_gpio_number(channel, &gpio))
@@ -509,22 +517,27 @@ static PyObject *py_wait_for_edge(PyObject *self, PyObject *args)
       return NULL;
    }
 
+   if (bouncetime <= 0 && bouncetime != -666)
+   {
+      PyErr_SetString(PyExc_ValueError, "Bouncetime must be greater than 0");
+      return NULL;
+   }
+
    if (check_gpio_priv())
       return NULL;
 
    Py_BEGIN_ALLOW_THREADS // disable GIL
-   result = blocking_wait_for_edge(gpio, edge);
+   result = blocking_wait_for_edge(gpio, edge, bouncetime);
    Py_END_ALLOW_THREADS   // enable GIL
 
    if (result == 0) {
       Py_INCREF(Py_None);
       return Py_None;
    } else if (result == 1) {
-      PyErr_SetString(PyExc_RuntimeError, "Edge detection events already enabled for this GPIO channel");
+      PyErr_SetString(PyExc_RuntimeError, "Conflicting edge detection events already exists for this GPIO channel");
       return NULL;
    } else {
-      sprintf(error, "Error #%d waiting for edge", result);
-      PyErr_SetString(PyExc_RuntimeError, error);
+      PyErr_SetString(PyExc_RuntimeError, "Error waiting for edge");
       return NULL;
    }
 
@@ -633,7 +646,7 @@ PyMethodDef rpi_gpio_methods[] = {
    {"remove_event_detect", py_remove_event_detect, METH_VARARGS, "Remove edge detection for a particular GPIO channel\nchannel - either board pin number or BCM number depending on which mode is set."},
    {"event_detected", py_event_detected, METH_VARARGS, "Returns True if an edge has occured on a given GPIO.  You need to enable edge detection using add_event_detect() first.\nchannel - either board pin number or BCM number depending on which mode is set."},
    {"add_event_callback", (PyCFunction)py_add_event_callback, METH_VARARGS | METH_KEYWORDS, "Add a callback for an event already defined using add_event_detect()\nchannel      - either board pin number or BCM number depending on which mode is set.\ncallback     - a callback function"},
-   {"wait_for_edge", py_wait_for_edge, METH_VARARGS, "Wait for an edge.\nchannel - either board pin number or BCM number depending on which mode is set.\nedge    - RISING, FALLING or BOTH"},
+   {"wait_for_edge", (PyCFunction)py_wait_for_edge, METH_VARARGS | METH_KEYWORDS, "Wait for an edge.\nchannel      - either board pin number or BCM number depending on which mode is set.\nedge         - RISING, FALLING or BOTH\n[bouncetime] - time allowed between calls to allow for switchbounce"},
    {"gpio_function", py_gpio_function, METH_VARARGS, "Return the current GPIO function (IN, OUT, PWM, SERIAL, I2C, SPI)\nchannel - either board pin number or BCM number depending on which mode is set."},
    {"setwarnings", py_setwarnings, METH_VARARGS, "Enable or disable warning messages"},
    {NULL, NULL, 0, NULL}
